package com.dhtmlx.xml2excel;
import java.io.File;
import java.io.IOException;

import javax.servlet.http.HttpServletResponse;


import jxl.Workbook;
import jxl.format.Alignment;
import jxl.format.Border;
import jxl.format.BorderLineStyle;
import jxl.format.Colour;
import jxl.format.VerticalAlignment;
import jxl.write.*;
import jxl.write.Number;
import jxl.write.biff.*;


public class ExcelWriter {
	private WritableWorkbook wb;
	private WritableSheet sheet;
	private ExcelColumn[][] cols;
	
	public int headerOffset = 0;
	public int scale = 150;
	public String pathToImgs = "";//optional, physical path
	
	public void generate(String xml, HttpServletResponse resp){
		ExcelXmlParser parser = new ExcelXmlParser();
		try {
			
			parser.setXML(xml);
			createExcel(resp);
			headerPrint(parser);
			rowsPrint(parser,resp);
			footerPrint(parser);
			insertHeader(parser,resp);
			insertFooter(parser,resp);
			outputExcel(resp);
			
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}
	
	private void createExcel(HttpServletResponse resp) throws IOException {
		wb = Workbook.createWorkbook(resp.getOutputStream());
		sheet = wb.createSheet("First Sheet", 0);
	}

	private void outputExcel(HttpServletResponse resp) throws IOException, WriteException {
		resp.setContentType("application/vnd.ms-excel");
		resp.setCharacterEncoding("UTF-8");
		resp.setHeader("Content-Disposition", "attachment;filename=grid.xls");
		resp.setHeader("Cache-Control", "max-age=0");
		wb.write();
		wb.close();
	}

	private void headerPrint(ExcelXmlParser parser) throws RowsExceededException, WriteException {
		cols = parser.getColumnsInfo();
		int widths[] = parser.getWidths();
		int sumWidth = 0;
		for (int i = 0; i < widths.length; i++) {
			sumWidth += widths[i];
		}
		for (int i = 0; i < cols.length; i++) {
			sheet.setRowView(i, 450);
			for (int j = 0; j < cols[i].length; j++) {
				sheet.setColumnView(j, widths[j]*scale/sumWidth);
				WritableFont font = new WritableFont(WritableFont.TAHOMA, 10, WritableFont.BOLD);
				WritableCellFormat f = new WritableCellFormat (font);
				f.setBackground(Colour.GRAY_25);
				f.setBorder(Border.ALL, BorderLineStyle.THIN, Colour.GRAY_50);
				f.setVerticalAlignment(VerticalAlignment.CENTRE);

				f.setAlignment(Alignment.CENTRE);
				String name = cols[i][j].getName();
				Label label = new Label(j, i, name, f);
				sheet.addCell(label);
			}
		}
		headerOffset = cols.length;
		for (int i = 0; i < cols.length; i++) {
			for (int j = 0; j < cols[i].length; j++) {
				int cspan = cols[i][j].getColspan();
				if (cspan > 0) {
					sheet.mergeCells(j, i, j + cspan - 1, i);
				}
				int rspan = cols[i][j].getRowspan();
				if (rspan > 0) {
					sheet.mergeCells(j, i, j, i + rspan - 1);
				}
			}
		}
	}

	private void footerPrint(ExcelXmlParser parser) throws RowsExceededException, WriteException {
		cols = parser.getFooterInfo();
		int widths[] = parser.getWidths();
		int sumWidth = 0;
		for (int i = 0; i < widths.length; i++) {
			sumWidth += widths[i];
		}
		for (int i = 0; i < cols.length; i++) {
			sheet.setRowView(i, 450);
			for (int j = 0; j < cols[i].length; j++) {
				sheet.setColumnView(j, widths[j]*scale/sumWidth);
				WritableFont font = new WritableFont(WritableFont.TAHOMA, 10, WritableFont.BOLD);
				WritableCellFormat f = new WritableCellFormat (font);
				f.setBackground(Colour.GRAY_25);
				f.setBorder(Border.ALL, BorderLineStyle.THIN, Colour.GRAY_50);
				f.setVerticalAlignment(VerticalAlignment.CENTRE);

				f.setAlignment(Alignment.CENTRE);
				String name = cols[i][j].getName();
				Label label = new Label(j, i + headerOffset, name, f);
				sheet.addCell(label);
			}
		}
		headerOffset = cols.length;
		for (int i = 0; i < cols.length; i++) {
			for (int j = 0; j < cols[i].length; j++) {
				int cspan = cols[i][j].getColspan();
				if (cspan > 0) {
					sheet.mergeCells(j, i, j + cspan - 1, i);
				}
				int rspan = cols[i][j].getRowspan();
				if (rspan > 0) {
					sheet.mergeCells(j, i, j, i + rspan - 1);
				}
			}
		}
	}
	
	private void rowsPrint(ExcelXmlParser parser, HttpServletResponse resp) throws WriteException, IOException {
		ExcelRow[] rows = parser.getGridContent();
		for (int i = 0; i < rows.length; i++) {
			String[] cells = rows[i].getCells();
			sheet.setRowView(i + headerOffset, 400);
			for (int j = 0; j < cells.length; j++) {
				WritableFont font = new WritableFont(WritableFont.TAHOMA, 10);
				WritableCellFormat f = new WritableCellFormat (font);
				if (i%2 == 1) {
					f.setBackground(Colour.LIGHT_GREEN);
				} else {
					f.setBackground(Colour.WHITE);
				}

				f.setBorder(Border.ALL, BorderLineStyle.THIN, Colour.GRAY_50);
				f.setVerticalAlignment(VerticalAlignment.CENTRE);
				String al = cols[0][j].getAlign();
				if (al.equalsIgnoreCase("left")) {
					f.setAlignment(Alignment.LEFT);
				} else {
					if (al.equalsIgnoreCase("right")) {
						f.setAlignment(Alignment.RIGHT);
					} else {
						f.setAlignment(Alignment.CENTRE);
					}
				}

				try {
					double name = Double.parseDouble(cells[j]);
					Number label = new Number(j, i + headerOffset, name, f);
					sheet.addCell(label);
				} catch (Exception e) {
					String name = cells[j];
					Label label = new Label(j, i + headerOffset, name, f);
					sheet.addCell(label);
				}
			}
		}
		headerOffset += rows.length;
	}

	private void insertHeader(ExcelXmlParser parser, HttpServletResponse resp) throws IOException, RowsExceededException {
		if (parser.getHeader() == true) {
			sheet.insertRow(0);
			sheet.setRowView(0, 5000);
			File imgFile = new File(pathToImgs + "/header.png");
			WritableImage img = new WritableImage(0, 0, cols[0].length, 1, imgFile);
			sheet.addImage(img);
		}
		headerOffset++;
	}

	private void insertFooter(ExcelXmlParser parser, HttpServletResponse resp) throws IOException, RowsExceededException {
		if (parser.getFooter() == true) {
			sheet.setRowView(headerOffset, 5000);
			File imgFile = new File(pathToImgs + "/footer.png");
			WritableImage img = new WritableImage(0, headerOffset, cols[0].length, 1, imgFile);
			sheet.addImage(img);
		}
	}
}
